@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use base qw(
     DateTime::Format::Natural::Aliases
-    DateTime::Format::Natural::Base
+    DateTime::Format::Natural::Calc
     DateTime::Format::Natural::Duration
     DateTime::Format::Natural::Formatted
     DateTime::Format::Natural::Helpers
@@ -18,7 +18,7 @@ use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '0.87';
+our $VERSION = '0.89';
 
 validation_options(
     on_fail => sub
@@ -64,7 +64,7 @@ sub _init
     $self->{Daytime} = $opts{daytime} || {};
 
     my $mod = __PACKAGE__.'::Lang::'.uc($self->{Lang});
-    eval "use $mod"; die $@ if $@;
+    eval "require $mod"; die $@ if $@;
 
     $self->{data} = $mod->__new();
     $self->{grammar_class} = $mod;
@@ -78,7 +78,7 @@ sub _init_check
         lang => {
             type => SCALAR,
             optional => true,
-            regex => qr!^(?:en)$!,
+            regex => qr!^(?:en)$!i,
         },
         format => {
             type => SCALAR,
@@ -123,9 +123,7 @@ sub _init_vars
 {
     my $self = shift;
 
-    foreach my $member (qw(modified postprocess)) {
-        delete $self->{$member};
-    }
+    delete @$self{qw(modified postprocess)};
 }
 
 sub parse_datetime
@@ -134,9 +132,9 @@ sub parse_datetime
 
     $self->_parse_init(@_);
 
-    $self->{Input_string} = $self->{Date_string};
+    $self->{input_string} = $self->{date_string};
 
-    my $date_string = $self->{Date_string};
+    my $date_string = $self->{date_string};
 
     $self->_rewrite_aliases(\$date_string);
     $date_string =~ tr/,//d;
@@ -173,7 +171,7 @@ sub parse_datetime
         );
         my $method = $methods{$prefix};
 
-        if (none { $unit =~ /${_}s?/ } @{$self->{data}->__units('ordered')}) {
+        if (none { $unit =~ /^${_}s?$/ } @{$self->{data}->__units('ordered')}) {
             $self->_set_failure;
             $self->_set_error("(invalid unit)");
             return $self->_get_datetime_object;
@@ -220,7 +218,7 @@ sub _parse_init
 {
     my $self = shift;
 
-    $self->_params_init(@_, { string => \$self->{Date_string} });
+    $self->_params_init(@_, { string => \$self->{date_string} });
 
     my $set_datetime = sub
     {
@@ -285,12 +283,10 @@ sub parse_datetime_duration
     $self->_post_duration(\@queue);
     $self->_restore_state;
 
-    foreach my $member (qw(duration formatted insert state)) {
-        delete $self->{$member};
-    }
+    delete @$self{qw(duration insert state)};
 
     @{$self->{traces}} = @traces;
-    $self->{Input_string} = $duration_string;
+    $self->{input_string} = $duration_string;
 
     return @queue;
 }
@@ -308,7 +304,7 @@ sub error
 
     return '' if $self->success;
 
-    my $error  = "'$self->{Input_string}' does not parse ";
+    my $error  = "'$self->{input_string}' does not parse ";
        $error .= $self->_get_error || '(perhaps you have some garbage?)';
 
     return $error;
@@ -341,7 +337,7 @@ sub _process
             my $valid_expression = true;
             my $definition = $expression->[0];
             my @positions = sort {$a <=> $b} keys %$definition;
-            my %regex_stack;
+            my (%first_stack, %rest_stack);
             foreach my $pos (@positions) {
                 if ($types->[$pos] eq 'SCALAR') {
                     if (defined $definition->{$pos}) {
@@ -355,9 +351,9 @@ sub _process
                     }
                 }
                 elsif ($types->[$pos] eq 'REGEXP') {
-                    local $1;
-                    if (${$self->_token($pos)} =~ $definition->{$pos}) {
-                        $regex_stack{$pos} = $1 if defined $1;
+                    if (my @captured = ${$self->_token($pos)} =~ $definition->{$pos}) {
+                        $first_stack{$pos} = shift @captured;
+                        $rest_stack{$pos} = [ @captured ];
                         next;
                     }
                     else {
@@ -375,7 +371,7 @@ sub _process
                 foreach my $check (@{$expression->[2]}) {
                     my @pos = @{$expression->[1][$i++]};
                     my $error;
-                    $valid_expression &= $check->(\%regex_stack, \@pos, \$error);
+                    $valid_expression &= $check->(\%first_stack, \%rest_stack, \@pos, \$error);
                     unless ($valid_expression) {
                         $self->_set_error("($error)");
                         last;
@@ -392,9 +388,9 @@ sub _process
                         $values[$c++] = ref $pos
                           ? $index eq 'VALUE'
                             ? $pos->{$index}
-                            : $self->SUPER::_helper($pos->{$index}, $regex_stack{$index})
-                          : exists $regex_stack{$index}
-                            ? $regex_stack{$index}
+                            : $self->SUPER::_helper($pos->{$index}, $first_stack{$index})
+                          : exists $first_stack{$index}
+                            ? $first_stack{$index}
                             : ${$self->_token($index)};
                     }
                     my $worker = "SUPER::$expression->[5]->[$i]";
